/*
    ettercap -- Inet -- Packet foundry

    Copyright (C) 2001  ALoR <alor@users.sourceforge.net>, NaGA <crwm@freemail.it>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

    $Id: ec_inet_forge.c,v 1.7 2002/05/10 20:09:15 alor Exp $
*/

#include "include/ec_main.h"
#include "include/ec_inet_structures.h"

// protos....

u_char * Inet_Forge_packet( u_short size );
int Inet_Forge_packet_destroy( u_char *buf );
int Inet_Forge_ethernet( u_char *buf, u_char *source, u_char *dest, u_short type );
int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da, u_long dip );
int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ident, u_short f_f, u_char proto );
int Inet_Forge_icmp( u_char *buf, u_char type, u_char code, u_char *data, int data_len );
int Inet_Forge_tcp( u_char *buf, u_short sp, u_short dp, u_long seq, u_long ack, u_char flags, u_char *data, int data_len );
int Inet_Forge_udp( u_char *buf, u_short sp, u_short dp, u_char *data, int data_len );
int Inet_Forge_Insert_TCPOpt( u_char *buf, u_char *options, int optlen );
u_short Inet_Forge_ChecksumIP(u_short *buffer, int size);
u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long IPsource, u_long IPdest);
u_long Inet_Forge_CRC(u_char *buffer, int len);

//---------------------------------------

u_char * Inet_Forge_packet( u_short size )
{

   u_char *buf;

   if (!size)
      size = MAX_PACKET_SIZE;

   buf = (u_char *)calloc(size+50, sizeof(u_char));
   if (buf == NULL)
      ERROR_MSG("calloc()");

   return (buf);
}


int Inet_Forge_packet_destroy( u_char *buf )
{

   if (buf != NULL)
   {
      free(buf);
      buf = NULL;
   }

   return 0;

}


int Inet_Forge_ethernet( u_char *buf, u_char *source, u_char *dest, u_short type )
{
   ETH_header eth;

   memcpy(eth.source_mac, source, ETHER_ADDR_LEN);  // source address
   memcpy(eth.dest_mac, dest, ETHER_ADDR_LEN);      // destination address
   eth.type = htons(type);                          // packet type

   memcpy(buf, &eth, sizeof(eth));

   return (sizeof(eth));
}


int Inet_Forge_arp( u_char *buf, u_short op, u_char *sa, u_long sip, u_char *da, u_long dip )
{
   ARP_header arp;

   arp.hw_type       = htons(ARPHRD_ETHER);    // hardware address type
   arp.proto_type    = htons(ETH_P_IP);        // protocol address type
   arp.ha_len        = 6;                      // hardware address length
   arp.pa_len        = 4;                      // protocol address length
   arp.opcode        = htons(op);              // opcode
   memcpy(arp.source_add, sa, 6);              // source mac
   memcpy(&arp.source_ip, &sip, 4);             // source ip
   memcpy(arp.dest_add, da, 6);                // dest mac
   memcpy(&arp.dest_ip, &dip, 4);               // dest ip

   memcpy(buf, &arp, sizeof(arp));

   return (sizeof(arp));
}



int Inet_Forge_ip( u_char *buf, u_long src, u_long dst, u_short len, u_short ident, u_short f_f, u_char proto )
{
   IP_header ip;

   ip.h_len           = 5;                             // 20 byte header
   ip.version         = 4;                             // version 4
   ip.tos             = 7;                             // IP tos     // we need 7 to enable GRE sniffing/injecting
   ip.t_len           = htons(IP_HEADER + len);        // total length
   ip.ident           = htons(ident);                  // IP ident
   ip.frag_and_flags  = htons(f_f);                    // fragmentation flags
   ip.ttl             = 125;                           // time to live
   ip.proto           = proto;                         // transport protocol
   ip.checksum        = 0;                             // checksum done in Inet_Forge_ChecksumIP
   ip.source_ip       = src;                           // source IP
   ip.dest_ip         = dst;                           // dest IP

   // checksumming...
   ip.checksum        = Inet_Forge_ChecksumIP( (u_short *)&ip, sizeof(IP_header) );

   memcpy(buf, &ip, sizeof(ip));

   return (sizeof(ip));
}


int Inet_Forge_icmp( u_char *buf, u_char type, u_char code, u_char *data, int data_len )
{
   ICMP_header *icmp;

   icmp = (ICMP_header *)buf;

   icmp->un.echo.seq = 1;             // seq
   icmp->un.echo.id  = htons(0xe77e); // ident   e77ercap :)
   icmp->code        = code;          // code
   icmp->type        = type;          // icmp type
   icmp->checksum    = 0;             // checksum

   if (data && data_len)
      memcpy(buf + ICMP_HEADER, data, data_len);

   // checksumming...
   icmp->checksum = Inet_Forge_ChecksumIP((u_short *)icmp, ICMP_HEADER + data_len );


   return (ICMP_HEADER + data_len);
}




int Inet_Forge_tcp( u_char *buf, u_short sp, u_short dp, u_long seq, u_long ack, u_char flags, u_char *data, int data_len )
{
   TCP_header *tcp;
   IP_header *ip;

   tcp = (TCP_header *)buf;

   tcp->source      = htons(sp);    // source port
   tcp->dest        = htons(dp);    // destination port
   tcp->seq         = htonl(seq);   // sequence number
   tcp->ack_seq     = htonl(ack);   // acknowledgement number
   tcp->flags       = flags;        // flags
   tcp->unused      = 0;            // UNUSED
   tcp->doff        = 5;            // 20 byte header
   tcp->window      = htons(32120); // window size
   tcp->checksum    = 0;            // checksum done in Inet_Forge_ChecksunTCP
   tcp->urg_ptr     = 0;            // urgent pointer


   if (data && data_len)
       memcpy(buf + TCP_HEADER, data, data_len);

   // checksumming...
   ip = (IP_header *)(buf - IP_HEADER);
   tcp->checksum    = Inet_Forge_Checksum( (u_short *)tcp, IPPROTO_TCP, TCP_HEADER+data_len, ip->source_ip, ip->dest_ip );

   return (TCP_HEADER + data_len);
}



int Inet_Forge_udp( u_char *buf, u_short sp, u_short dp, u_char *data, int data_len )
{
   UDP_header *udp;
   IP_header *ip;

   udp = (UDP_header *)buf;

   udp->source      = htons(sp);                         // source port
   udp->dest        = htons(dp);                         // destination port
   udp->len         = htons(data_len + UDP_HEADER);      // data length
   udp->checksum    = 0;                                 // checksum


   if (data && data_len)
       memcpy(buf + UDP_HEADER, data, data_len);

   // checksumming...
   ip = (IP_header *)(buf - IP_HEADER);
   udp->checksum    = Inet_Forge_Checksum( (u_short *)udp, IPPROTO_UDP, UDP_HEADER+data_len, ip->source_ip, ip->dest_ip );

   return (UDP_HEADER + data_len);
}


int Inet_Forge_Insert_TCPOpt( u_char *buf, u_char *options, int optlen )
{
   IP_header *ip;
   TCP_header *tcp;
   int total_len, datalen;
   u_char *data;

   ip   = (IP_header *)(buf - IP_HEADER);
   tcp  = (TCP_header *)(buf);
   data = (char *)((int)tcp + tcp->doff * 4);

   total_len = ip->t_len + ETH_HEADER;

   datalen = (int)ip + ntohs(ip->t_len) - (int)data;
   memmove(data + optlen, data, datalen);
   memcpy(data, options, optlen);

   ip->t_len = htons( ntohs(ip->t_len) + optlen );
   ip->checksum = 0;
   ip->checksum = Inet_Forge_ChecksumIP( (u_short *)ip, IP_HEADER );

   tcp->doff = tcp->doff + optlen / 4;
   tcp->checksum = 0;
   tcp->checksum = Inet_Forge_Checksum( (u_short *)tcp, IPPROTO_TCP, tcp->doff*4 + datalen, ip->source_ip, ip->dest_ip );

   return (tcp->doff*4 + datalen);
}




u_short Inet_Forge_ChecksumIP(u_short *buffer, int size)
{
   u_long cksum = 0;

   while(size >1)
   {
      cksum += *buffer++;
      size -= sizeof(u_short);
   }

   if (size) cksum += *(u_char*)buffer;

   cksum = (cksum >> 16) + (cksum & 0xffff);
   cksum += (cksum >> 16);
   return (u_short)(~cksum);

}



u_short Inet_Forge_Checksum(u_short *buffer, u_short proto, u_short size, u_long IPsource, u_long IPdest)
{
   u_long cksum = 0;
   u_short len;

   len = size;

   while( size > 1 )
   {
      cksum += *buffer++;
      size -= sizeof(u_short);
   }

   cksum += IPsource >> 16;
   cksum += IPsource & 0xffff;
   cksum += IPdest >> 16;
   cksum += IPdest & 0xffff;
   cksum += htons(proto);
   cksum += htons(len);

   if ( size ) cksum += *(u_char*)buffer;

   cksum = (cksum >> 16) + (cksum & 0xffff);
   cksum += (cksum >> 16);
   return (u_short)(~cksum);

}


u_long Inet_Forge_CRC(u_char *buffer, int len)
{
    unsigned long crc_32_tab[] = {
    0x00000000, 0x77073096, 0xee0e612c, 0x990951ba, 0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
    0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988, 0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
    0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de, 0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
    0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec, 0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
    0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172, 0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
    0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940, 0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
    0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116, 0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
    0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924, 0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
    0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a, 0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
    0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818, 0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
    0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e, 0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
    0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c, 0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
    0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2, 0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
    0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0, 0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
    0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086, 0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
    0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4, 0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
    0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a, 0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
    0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8, 0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
    0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe, 0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
    0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc, 0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
    0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252, 0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
    0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60, 0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
    0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236, 0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
    0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04, 0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
    0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a, 0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
    0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38, 0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
    0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e, 0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
    0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c, 0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
    0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2, 0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
    0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0, 0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
    0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6, 0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
    0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94, 0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
    };
    u_long crc;
    u_long i;

    crc = 0x0;
    for(i=0; i<len; i++)
    crc = crc_32_tab[(crc^buffer[i]) &0xff] ^ (crc>>8);

    return crc;
}


/* EOF */

// vim:ts=3:expandtab

